"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDepositIdAtBlock = exports.getBlockRangeForDepositId = void 0;
var tslib_1 = require("tslib");
function getBlockRangeForDepositId(targetDepositId, initLow, initHigh, maxSearches, spokePool, deploymentBlock) {
    if (deploymentBlock === void 0) { deploymentBlock = 0; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var queriedIds, _getDepositIdAtBlock, _a, mostRecentBlockNumber, highestPossibleDepositIdInRange, lowestPossibleDepositIdInRange, low, high, searches, mid, midDepositId, accountedIdByMidBlock;
        var _this = this;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    queriedIds = {};
                    _getDepositIdAtBlock = function (blockNumber) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var _a, _b;
                        return tslib_1.__generator(this, function (_c) {
                            switch (_c.label) {
                                case 0:
                                    if (!(queriedIds[blockNumber] === undefined)) return [3, 2];
                                    _a = queriedIds;
                                    _b = blockNumber;
                                    return [4, getDepositIdAtBlock(spokePool, blockNumber)];
                                case 1:
                                    _a[_b] = _c.sent();
                                    _c.label = 2;
                                case 2: return [2, queriedIds[blockNumber]];
                            }
                        });
                    }); };
                    return [4, Promise.all([
                            spokePool.provider.getBlockNumber(),
                            _getDepositIdAtBlock(initHigh),
                            _getDepositIdAtBlock(Math.max(deploymentBlock, initLow - 1)),
                        ])];
                case 1:
                    _a = _b.sent(), mostRecentBlockNumber = _a[0], highestPossibleDepositIdInRange = _a[1], lowestPossibleDepositIdInRange = _a[2];
                    initHigh = Math.min(initHigh, mostRecentBlockNumber);
                    [
                        [initLow <= initHigh, "Binary search failed because low > high"],
                        [initLow >= deploymentBlock, "Binary search failed because low must be >= deploymentBlock"],
                        [maxSearches > 0, "maxSearches must be > 0"],
                        [deploymentBlock >= 0, "deploymentBlock must be >= 0"],
                    ].forEach(function (_a) {
                        var condition = _a[0], errorMessage = _a[1];
                        if (!condition) {
                            throw new Error(errorMessage);
                        }
                    });
                    if (highestPossibleDepositIdInRange <= targetDepositId) {
                        throw new Error("Target depositId is greater than the initial high block (".concat(targetDepositId, " > ").concat(initHigh, ")"));
                    }
                    if (lowestPossibleDepositIdInRange > targetDepositId) {
                        throw new Error("Target depositId is less than the initial low block (".concat(targetDepositId, " > ").concat(initLow, ")"));
                    }
                    low = initLow;
                    high = initHigh;
                    searches = 0;
                    _b.label = 2;
                case 2:
                    mid = Math.floor((low + high) / 2);
                    return [4, _getDepositIdAtBlock(mid)];
                case 3:
                    midDepositId = _b.sent();
                    accountedIdByMidBlock = midDepositId - 1;
                    if (targetDepositId <= accountedIdByMidBlock) {
                        high = mid;
                    }
                    else {
                        low = mid + 1;
                    }
                    _b.label = 4;
                case 4:
                    if (++searches <= maxSearches && low < high) return [3, 2];
                    _b.label = 5;
                case 5:
                    if (low > high) {
                        throw new Error("Binary search failed (".concat(low, " > ").concat(high, "). SHOULD NEVER HAPPEN (but here we are)"));
                    }
                    return [2, { low: low, high: high }];
            }
        });
    });
}
exports.getBlockRangeForDepositId = getBlockRangeForDepositId;
function getDepositIdAtBlock(contract, blockTag) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var depositIdAtBlock;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, contract.numberOfDeposits({ blockTag: blockTag })];
                case 1:
                    depositIdAtBlock = _a.sent();
                    if (!Number.isInteger(depositIdAtBlock) || depositIdAtBlock < 0) {
                        throw new Error("Invalid deposit count");
                    }
                    return [2, depositIdAtBlock];
            }
        });
    });
}
exports.getDepositIdAtBlock = getDepositIdAtBlock;
//# sourceMappingURL=SpokeUtils.js.map