import { __awaiter, __generator } from "tslib";
import { isDefined } from "./TypeGuards";
import { getCurrentTime } from "./TimeUtils";
// Archive requests typically commence at 128 blocks past the head of the chain.
// Round down to 120 blocks to avoid slipping into archive territory.
var defaultBlockRange = 120;
// Retain computations for 15 minutes.
var cacheTTL = 60 * 15;
var blockTimes = {};
/**
 * @description Compute the average block time over a block range.
 * @returns Average number of seconds per block.
 */
export function averageBlockTime(provider, _a) {
    var _b = _a === void 0 ? {} : _a, latestBlockNumber = _b.latestBlockNumber, blockRange = _b.blockRange;
    return __awaiter(this, void 0, void 0, function () {
        var chainId, cache, now, _c, _d, firstBlock, lastBlock, average;
        return __generator(this, function (_e) {
            switch (_e.label) {
                case 0: return [4 /*yield*/, provider.getNetwork()];
                case 1:
                    chainId = (_e.sent()).chainId;
                    cache = blockTimes[chainId];
                    now = getCurrentTime();
                    if (isDefined(cache) && now < cache.timestamp + cacheTTL) {
                        return [2 /*return*/, { average: cache.average, blockRange: cache.blockRange }];
                    }
                    if (!(latestBlockNumber !== null && latestBlockNumber !== void 0)) return [3 /*break*/, 2];
                    _c = latestBlockNumber;
                    return [3 /*break*/, 4];
                case 2: return [4 /*yield*/, provider.getBlockNumber()];
                case 3:
                    _c = (latestBlockNumber = _e.sent());
                    _e.label = 4;
                case 4:
                    _c;
                    blockRange !== null && blockRange !== void 0 ? blockRange : (blockRange = defaultBlockRange);
                    return [4 /*yield*/, Promise.all([
                            provider.getBlock(latestBlockNumber - blockRange),
                            provider.getBlock(latestBlockNumber),
                        ])];
                case 5:
                    _d = _e.sent(), firstBlock = _d[0], lastBlock = _d[1];
                    average = (lastBlock.timestamp - firstBlock.timestamp) / blockRange;
                    blockTimes[chainId] = { timestamp: now, average: average, blockRange: blockRange };
                    return [2 /*return*/, { average: average, blockRange: blockRange }];
            }
        });
    });
}
//# sourceMappingURL=BlockUtils.js.map